@@ -2,7 +2,7 @@
  *   prompt.c  --  This file is part of GNU nano.                         *
  *                                                                        *
  *   Copyright (C) 1999-2011, 2013-2020 Free Software Foundation, Inc.    *
- *   Copyright (C) 2016, 2018 Benno Schulenberg                           *
+ *   Copyright (C) 2016, 2018, 2020 Benno Schulenberg                     *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
  *   it under the terms of the GNU General Public License as published    *
@@ -57,9 +57,18 @@ void do_statusbar_next_word(void)
 			 * and if we've already seen a word, then it's a word end. */
 			if (is_word_char(answer + typing_x, FALSE))
 				seen_word = TRUE;
+#ifdef ENABLE_UTF8
+			else if (is_zerowidth(answer + typing_x))
+				; /* skip */
+#endif
 			else if (seen_word)
 				break;
 		} else {
+#ifdef ENABLE_UTF8
+			if (is_zerowidth(answer + typing_x))
+				; /* skip */
+			else
+#endif
 			/* If this is not a word character, then it's a separator; else
 			 * if we've already seen a separator, then it's a word start. */
 			if (!is_word_char(answer + typing_x, FALSE))
@@ -81,6 +90,10 @@ void do_statusbar_prev_word(void)
 
 		if (is_word_char(answer + typing_x, FALSE))
 			seen_a_word = TRUE;
+#ifdef ENABLE_UTF8
+		else if (is_zerowidth(answer + typing_x))
+			; /* skip */
+#endif
 		else if (seen_a_word) {
 			/* This is space now: we've overshot the start of the word. */
 			step_forward = TRUE;
@@ -97,15 +110,25 @@ void do_statusbar_prev_word(void)
 /* Move left one character in the answer. */
 void do_statusbar_left(void)
 {
-	if (typing_x > 0)
+	if (typing_x > 0) {
 		typing_x = step_left(answer, typing_x);
+#ifdef ENABLE_UTF8
+		while (typing_x > 0 && is_zerowidth(answer + typing_x))
+			typing_x = step_left(answer, typing_x);
+#endif
+	}
 }
 
 /* Move right one character in the answer. */
 void do_statusbar_right(void)
 {
-	if (answer[typing_x] != '\0')
+	if (answer[typing_x] != '\0') {
 		typing_x = step_right(answer, typing_x);
+#ifdef ENABLE_UTF8
+		while (answer[typing_x] != '\0' && is_zerowidth(answer + typing_x))
+			typing_x = step_right(answer, typing_x);
+#endif
+	}
 }
 
 /* Delete one character in the answer. */
@@ -116,6 +139,10 @@ void do_statusbar_delete(void)
 
 		memmove(answer + typing_x, answer + typing_x + charlen,
 						strlen(answer) - typing_x - charlen + 1);
+#ifdef ENABLE_UTF8
+		if (is_zerowidth(answer + typing_x))
+			do_statusbar_delete();
+#endif
 	}
 }
 
@@ -123,8 +150,10 @@ void do_statusbar_delete(void)
 void do_statusbar_backspace(void)
 {
 	if (typing_x > 0) {
+		size_t was_x = typing_x;
+
 		typing_x = step_left(answer, typing_x);
-		do_statusbar_delete();
+		memmove(answer + typing_x, answer + was_x, strlen(answer) - was_x + 1);
 	}
 }
 
@@ -378,10 +407,6 @@ void draw_the_promptbar(void)
 
 	wattroff(bottomwin, interface_color_pair[TITLE_BAR]);
 
-	/* Work around a cursor-misplacement bug in VTEs. */
-	wmove(bottomwin, 0, 0);
-	wrefresh(bottomwin);
-
 	/* Place the cursor at the right spot. */
 	column = base + wideness(answer, typing_x);
 	wmove(bottomwin, 0, column - get_statusbar_page_start(base, column));
